@@ -1,6 +1,6 @@
 /* FAudio - XAudio Reimplementation for FNA
  *
- * Copyright (c) 2011-2020 Ethan Lee, Luigi Auriemma, and the MonoGame Team
+ * Copyright (c) 2011-2024 Ethan Lee, Luigi Auriemma, and the MonoGame Team
  *
  * This software is provided 'as-is', without any express or implied warranty.
  * In no event will the authors be held liable for any damages arising from
@@ -28,19 +28,23 @@
 #include "FAPOBase.h"
 #include <stdarg.h>
 
-#ifdef FAUDIO_UNKNOWN_PLATFORM
+#ifdef FAUDIO_WIN32_PLATFORM
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <malloc.h>
 #include <math.h>
 #include <assert.h>
 #include <inttypes.h>
 
+#define WIN32_LEAN_AND_MEAN
+#include <windows.h>
+
 #define FAudio_malloc malloc
 #define FAudio_realloc realloc
 #define FAudio_free free
-#define FAudio_alloca(x) alloca(uint8_t, x)
-#define FAudio_dealloca(x) dealloca(x)
+#define FAudio_alloca(x) alloca(x)
+#define FAudio_dealloca(x) (void)(x)
 #define FAudio_zero(ptr, size) memset(ptr, '\0', size)
 #define FAudio_memset(ptr, val, size) memset(ptr, val, size)
 #define FAudio_memcpy(dst, src, size) memcpy(dst, src, size)
@@ -49,7 +53,8 @@
 
 #define FAudio_strlen(ptr) strlen(ptr)
 #define FAudio_strcmp(str1, str2) strcmp(str1, str2)
-#define FAudio_strlcpy(ptr1, ptr2, size) strlcpy(ptr1, ptr2, size)
+#define FAudio_strncmp(str1, str2, size) strncmp(str1, str2, size)
+#define FAudio_strlcpy(ptr1, ptr2, size) lstrcpynA(ptr1, ptr2, size)
 
 #define FAudio_pow(x, y) pow(x, y)
 #define FAudio_log(x) log(x)
@@ -76,13 +81,37 @@
 #define FAudio_assert assert
 #define FAudio_snprintf snprintf
 #define FAudio_vsnprintf vsnprintf
-#define FAudio_Log(msg) fprintf(stderr, "%s\n", msg)
 #define FAudio_getenv getenv
 #define FAudio_PRIu64 PRIu64
 #define FAudio_PRIx64 PRIx64
+
+extern void FAudio_Log(char const *msg);
+
+/* FIXME: Assuming little-endian! */
+#define FAudio_swap16LE(x) (x)
+#define FAudio_swap16BE(x) \
+	((x >> 8)	& 0x00FF) | \
+	((x << 8)	& 0xFF00)
+#define FAudio_swap32LE(x) (x)
+#define FAudio_swap32BE(x) \
+	((x >> 24)	& 0x000000FF) | \
+	((x >> 8)	& 0x0000FF00) | \
+	((x << 8)	& 0x00FF0000) | \
+	((x << 24)	& 0xFF000000)
+#define FAudio_swap64LE(x) (x)
+#define FAudio_swap64BE(x) \
+	((x >> 32)	& 0x00000000000000FF) | \
+	((x >> 24)	& 0x000000000000FF00) | \
+	((x >> 16)	& 0x0000000000FF0000) | \
+	((x >> 8)	& 0x00000000FF000000) | \
+	((x << 8)	& 0x000000FF00000000) | \
+	((x << 16)	& 0x0000FF0000000000) | \
+	((x << 24)	& 0x00FF000000000000) | \
+	((x << 32)	& 0xFF00000000000000)
 #else
 #include <SDL_stdinc.h>
 #include <SDL_assert.h>
+#include <SDL_endian.h>
 #include <SDL_log.h>
 
 #define FAudio_malloc SDL_malloc
@@ -98,6 +127,7 @@
 
 #define FAudio_strlen(ptr) SDL_strlen(ptr)
 #define FAudio_strcmp(str1, str2) SDL_strcmp(str1, str2)
+#define FAudio_strncmp(str1, str2, size) SDL_strncmp(str1, str1, size)
 #define FAudio_strlcpy(ptr1, ptr2, size) SDL_strlcpy(ptr1, ptr2, size)
 
 #define FAudio_pow(x, y) SDL_pow(x, y)
@@ -141,6 +171,13 @@
 #define FAudio_getenv SDL_getenv
 #define FAudio_PRIu64 SDL_PRIu64
 #define FAudio_PRIx64 SDL_PRIx64
+
+#define FAudio_swap16LE(x) SDL_SwapLE16(x)
+#define FAudio_swap16BE(x) SDL_SwapBE16(x)
+#define FAudio_swap32LE(x) SDL_SwapLE32(x)
+#define FAudio_swap32BE(x) SDL_SwapBE32(x)
+#define FAudio_swap64LE(x) SDL_SwapLE64(x)
+#define FAudio_swap64BE(x) SDL_SwapBE64(x)
 #endif
 
 /* Easy Macros */
@@ -173,6 +210,15 @@
 #define restrict
 #endif
 
+/* Alignment macro for gcc/clang/msvc */
+#if defined(__clang__) || defined(__GNUC__)
+#define ALIGN(type, boundary) type __attribute__((aligned(boundary)))
+#elif defined(_MSC_VER)
+#define ALIGN(type, boundary) __declspec(align(boundary)) type
+#else
+#define ALIGN(type, boundary) type
+#endif
+
 /* Threading Types */
 
 typedef void* FAudioThread;
@@ -249,10 +295,8 @@ typedef void (FAUDIOCALL * FAudioMixCallback)(
 	uint32_t toMix,
 	uint32_t srcChans,
 	uint32_t dstChans,
-	float baseVolume,
 	float *restrict srcData,
 	float *restrict dstData,
-	float *restrict channelVolume,
 	float *restrict coefficients
 );
 
@@ -288,13 +332,13 @@ void FAudio_OPERATIONSET_QueueSetEffectParameters(
 );
 void FAudio_OPERATIONSET_QueueSetFilterParameters(
 	FAudioVoice *voice,
-	const FAudioFilterParameters *pParameters,
+	const FAudioFilterParametersEXT *pParameters,
 	uint32_t OperationSet
 );
 void FAudio_OPERATIONSET_QueueSetOutputFilterParameters(
 	FAudioVoice *voice,
 	FAudioVoice *pDestinationVoice,
-	const FAudioFilterParameters *pParameters,
+	const FAudioFilterParametersEXT *pParameters,
 	uint32_t OperationSet
 );
 void FAudio_OPERATIONSET_QueueSetVolume(
@@ -396,11 +440,13 @@ struct FAudioVoice
 
 	FAudioVoiceSends sends;
 	float **sendCoefficients;
+	float **mixCoefficients;
 	FAudioMixCallback *sendMix;
-	FAudioFilterParameters *sendFilter;
+	FAudioFilterParametersEXT *sendFilter;
 	FAudioFilterState **sendFilterState;
 	struct
 	{
+		FAPOBufferFlags state;
 		uint32_t count;
 		FAudioEffectDescriptor *desc;
 		void **parameters;
@@ -408,7 +454,7 @@ struct FAudioVoice
 		uint8_t *parameterUpdates;
 		uint8_t *inPlaceProcessing;
 	} effects;
-	FAudioFilterParameters filter;
+	FAudioFilterParametersEXT filter;
 	FAudioFilterState *filterState;
 	FAudioMutex sendLock;
 	FAudioMutex effectLock;
@@ -434,10 +480,10 @@ struct FAudioVoice
 			uint64_t curBufferOffsetDec;
 			uint32_t curBufferOffset;
 
-			/* FFmpeg */
-#ifdef HAVE_FFMPEG
-			struct FAudioFFmpeg *ffmpeg;
-#endif /* HAVE_FFMPEG*/
+			/* WMA decoding */
+#ifdef HAVE_WMADEC
+			struct FAudioWMADEC *wmadec;
+#endif /* HAVE_WMADEC*/
 
 			/* Read-only */
 			float maxFreqRatio;
@@ -452,6 +498,7 @@ struct FAudioVoice
 			uint8_t newBuffer;
 			uint64_t totalSamples;
 			FAudioBufferEntry *bufferList;
+			FAudioBufferEntry *flushList;
 			FAudioMutex bufferLock;
 		} src;
 		struct
@@ -492,13 +539,15 @@ void FAudio_INTERNAL_InsertSubmixSorted(
 );
 void FAudio_INTERNAL_UpdateEngine(FAudio *audio, float *output);
 void FAudio_INTERNAL_ResizeDecodeCache(FAudio *audio, uint32_t size);
-void FAudio_INTERNAL_ResizeResampleCache(FAudio *audio, uint32_t size);
-void FAudio_INTERNAL_ResizeEffectChainCache(FAudio *audio, uint32_t samples);
 void FAudio_INTERNAL_AllocEffectChain(
 	FAudioVoice *voice,
 	const FAudioEffectChain *pEffectChain
 );
 void FAudio_INTERNAL_FreeEffectChain(FAudioVoice *voice);
+uint32_t FAudio_INTERNAL_VoiceOutputFrequency(
+	FAudioVoice *voice,
+	const FAudioVoiceSends *pSendList
+);
 extern const float FAUDIO_INTERNAL_MATRIX_DEFAULTS[8][8][64];
 
 /* Debug */
@@ -656,15 +705,15 @@ extern void (*FAudio_INTERNAL_Amplify)(
 	float volume
 );
 
+extern FAudioMixCallback FAudio_INTERNAL_Mix_Generic;
+
 #define MIX_FUNC(type) \
 	extern void FAudio_INTERNAL_Mix_##type##_Scalar( \
 		uint32_t toMix, \
 		uint32_t srcChans, \
 		uint32_t dstChans, \
-		float baseVolume, \
 		float *restrict srcData, \
 		float *restrict dstData, \
-		float *restrict channelVolume, \
 		float *restrict coefficients \
 	);
 MIX_FUNC(Generic)
@@ -697,18 +746,15 @@ DECODE_FUNC(PCM32F)
 DECODE_FUNC(MonoMSADPCM)
 DECODE_FUNC(StereoMSADPCM)
 DECODE_FUNC(WMAERROR)
-#ifdef HAVE_FFMPEG
-DECODE_FUNC(FFMPEG)
-#endif /* HAVE_FFMPEG */
 #undef DECODE_FUNC
 
-/* FFmpeg */
+/* WMA decoding */
 
-#ifdef HAVE_FFMPEG
-uint32_t FAudio_FFMPEG_init(FAudioSourceVoice *pSourceVoice, uint32_t type);
-void FAudio_FFMPEG_free(FAudioSourceVoice *voice);
-void FAudio_FFMPEG_reset(FAudioSourceVoice *voice);
-#endif /* HAVE_FFMPEG */
+#ifdef HAVE_WMADEC
+uint32_t FAudio_WMADEC_init(FAudioSourceVoice *pSourceVoice, uint32_t type);
+void FAudio_WMADEC_free(FAudioSourceVoice *voice);
+void FAudio_WMADEC_end_buffer(FAudioSourceVoice *voice);
+#endif /* HAVE_WMADEC */
 
 /* Platform Functions */
 
@@ -768,7 +814,8 @@ static inline uint32_t GetMask(uint16_t channels)
 static inline void WriteWaveFormatExtensible(
 	FAudioWaveFormatExtensible *fmt,
 	int channels,
-	int samplerate
+	int samplerate,
+	const FAudioGUID *subformat
 ) {
 	FAudio_assert(fmt != NULL);
 	fmt->Format.wBitsPerSample = 32;
@@ -786,7 +833,7 @@ static inline void WriteWaveFormatExtensible(
 	fmt->Format.cbSize = sizeof(FAudioWaveFormatExtensible) - sizeof(FAudioWaveFormatEx);
 	fmt->Samples.wValidBitsPerSample = 32;
 	fmt->dwChannelMask = GetMask(fmt->Format.nChannels);
-	FAudio_memcpy(&fmt->SubFormat, &DATAFORMAT_SUBTYPE_IEEE_FLOAT, sizeof(FAudioGUID));
+	FAudio_memcpy(&fmt->SubFormat, subformat, sizeof(FAudioGUID));
 }
 
 /* Resampling */
